import numpy as np
import pylab

# Functia de cost: evalueaza gradul de precizie al modelului prin J(theta)
def compute_cost(x, y, theta):
	predictions = x.dot(theta).flatten()
	sqErrors = (predictions - y) ** 2
	J = (1.0 / (2.0*m)) * sqErrors.sum()
	return J

# Algoritmul iterativ de minimizare a gradientului
def gradient_descent(x, y, theta, alpha, num_iters):
	m = y.size
	n = len(x[0])
	J_history = np.zeros(shape=(num_iters, 1))
	theta_history = np.zeros(shape=(num_iters, n))

	for i in range(num_iters):
		predictions = x.dot(theta).flatten()
		for j in range(n):
			errors = (predictions - y) * x[:, j]
			theta[j] = theta[j] - alpha * (1./m) * errors.sum()

		J_history[i, 0] = compute_cost(x, y, theta)
		theta_history[i, :] = theta

	return theta, J_history, theta_history

# Incarcarea caracteristicilor de intrare (features)
features = np.loadtxt('ex3x.txt', delimiter=',')
# Incarcarea etichetelor (labels)
y = np.loadtxt('ex3y.txt', delimiter=',')

# Afisarea distributiei datelor
pylab.scatter(features, y, marker='o', c='b')
pylab.xlabel('Varsta [ani]')
pylab.ylabel('Inaltime [m]')
pylab.show()

'''
Parametrii algoritmului de regresie liniara
'''
# Numarul de caracteristici (la care se adauga x_0 = 1)
n = 2
# Numarul de exemple de antrenare
m = y.size

# Parametrii algoritmului de minimizare a gradientului
iterations = 1000
alpha = 0.07 # Rata de invatare

# Initializarea parametrilor modelului cu zero
theta = np.zeros(n)

# Crearea matricei caracteristicilor (ce contine o prima coloana de 1)
x = np.ones(shape=(m, n))
x[:, 1] = features # Adaugarea caracteristicilor (features) incarcate

# Aplicarea algoritmului de minimizare a gradientului
theta, J_history, theta_history = gradient_descent(x, y, theta, alpha, iterations)

print(J_history)

# Predictie pentru valorile de intrare [1, 3.5] si [1, 7]
test1 = np.array([1, 3.5])
test2 = np.array([1, 7])

print('Predictia inaltimii pentru varsta de 3 ani jumate: ', test1.dot(theta))
print('Predictia inaltimii pentru varsta de 7 ani: ', test2.dot(theta))


'''
Afisarea conturului functiei de cost
'''
# Gridul peste care vom calcula J
theta0_vals = pylab.linspace(-10, 10, 100)
theta1_vals = pylab.linspace(-1, 4, 100)

# Initializarea valorilor lui J cu zero
J_vals = np.zeros(shape=(theta0_vals.size, theta1_vals.size))

# Calculul valorilor lui J
for t1, element in enumerate(theta0_vals):
    for t2, element2 in enumerate(theta1_vals):
        thetaT = np.zeros(shape=(2, 1))
        thetaT[0][0] = element
        thetaT[1][0] = element2
        J_vals[t1, t2] = compute_cost(x, y, thetaT)

# Desenarea contururilor
J_vals = J_vals.T
# Afosarea lui J pentru 15 contururi pozitionate la distante logaritmice intre 0.01 si 100
pylab.contour(theta0_vals, theta1_vals, J_vals, pylab.logspace(-2, 3, 20))
pylab.xlabel('theta_0')
pylab.ylabel('theta_1')
pylab.scatter(theta[0], theta[1], marker='o', c='r')
pylab.show()
